package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesReworkTaskService;
import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesShiftService;
import cn.estsh.i3plus.ext.mes.pcn.apiservice.util.MesPcnException;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesReworkTaskModel;
import cn.estsh.i3plus.ext.mes.pcn.pojo.model.MesReworkTaskRequestModel;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspection;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspectionDetail;
import cn.estsh.i3plus.pojo.mes.bean.rework.MesReworkTask;
import cn.estsh.i3plus.pojo.mes.bean.rework.MesReworkTaskDetail;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;


/**
 * @Description: 查询返工单
 * @Author: zxw
 * @Date: 2024/5/21 14:16
 * @Modify:
 */
@Service
@Slf4j
public class MesReworkTaskServiceImpl implements IMesReworkTaskService {

    private static final Logger LOGGER = LoggerFactory.getLogger(MesReworkTaskServiceImpl.class);


    @Autowired
    private MesReworkTaskRepository mesReworkTaskRepository;
    @Autowired
    private MesPartTypePictureRepository mesPartTypePictureRepository;

    @Autowired
    private MesPartRepository mesPartRepository;

    @Autowired
    private MesPartInspectionRepository mesPartInspectionRepository;

    @Autowired
    private MesPartInspectionDetailRepository mesPartInspectionDetailRepository;

    @Autowired
    private MesReworkTaskDetailRepository mesReworkTaskDetailRepository;

    @Autowired
    private MesProductionAssemblyRepository mesProductionAssemblyRepository;

    @Autowired
    private MesProduceSnRepository mesProduceSnRepository;

    @Override
    public ListPager<MesReworkTask> queryReworkTask(MesReworkTask mesReworkTask, Pager pager) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(AuthUtil.getOrganizeCode());
        if (mesReworkTask.getSn() != null) {
            DdlPreparedPack.getStringEqualPack(mesReworkTask.getSn(), "sn", packBean);
        }
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.REWORK_TASK_STATUS.CREATE.getValue(), "status", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"createDatetime"}, packBean);

        pager = PagerHelper.getPager(pager, mesReworkTaskRepository.findByHqlWhereCount(packBean));

        List<MesReworkTask> resultList = mesReworkTaskRepository.findByHqlWherePage(packBean, pager);

        return new ListPager<>(resultList, pager);
    }


    @Override
    public MesReworkTaskModel queryReworkTaskModel(MesReworkTaskRequestModel requestModel){
        /**
         * 获取返工单信息
         */
        MesReworkTask res = getMesReworkTask(requestModel);

        /**
         * 获取返工单信息
         */
        List<MesReworkTaskDetail> reworkTaskDetails = getMesReworkTaskDetail(res);
        /**
         * 获取零件图片信息
         */
        MesPartTypePicture mesPartTypePicture = getMesPartTypePicture(res);
        /**
         * 获取缺陷记录
         */
        MesPartInspection mesPartInspection = getMesPartInspection(res);

        List<MesPartInspectionDetail> mesPartInspectionDetails = null;
        if (!CollectionUtils.isEmpty(reworkTaskDetails)) {
            /**
             * 获取零件缺陷明细列表
             */
            mesPartInspectionDetails = getMesPartInspectionDetail(reworkTaskDetails);
        }
        /**
         * 组装返回信息
         */
        MesReworkTaskModel mesReworkTaskModel = MesReworkTaskModel.builder()
                .reworkOrder(res.getReworkOrder())
                .sn(res.getSn())
                .mesPartTypePicture(mesPartTypePicture)
                .mesPartInspection(mesPartInspection)
                .mesPartInspectionDetails(mesPartInspectionDetails).build();


        return mesReworkTaskModel;
    }

    @Override
    public void reworkRecord(long detailId) {

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(AuthUtil.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(detailId, "partInspectionDetailId", packBean);

        MesReworkTaskDetail mesReworkTaskDetail = mesReworkTaskDetailRepository.getByProperty(packBean);
        if (mesReworkTaskDetail == null) {
            MesPcnException.throwFlowException("返工单详情不存在");
        }
        mesReworkTaskDetail.setStatus(MesExtEnumUtil.REWORK_TASK_DETAIL_STATUS.REWORK_TASK_DETAIL_STATUS_20.getValue());
        mesReworkTaskDetailRepository.save(mesReworkTaskDetail);

    }


    @Override
    public List<MesProductionAssembly> assemblyQuery(MesReworkTaskRequestModel requestModel) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(requestModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(requestModel.getCustSn(), "custSn", packBean);
        //DdlPreparedPack.getStringEqualPack(requestModel.getPartNo(), "partNo", packBean);
        if (!StringUtils.isEmpty(requestModel.getAssemblyPartNo())) {
            DdlPreparedPack.getStringEqualPack(requestModel.getAssemblyPartNo(), "assemblyPartNo", packBean);
        }
        if (!Objects.isNull(requestModel.getAssemblyPartStatus())) {
            DdlPreparedPack.getNumEqualPack(requestModel.getAssemblyPartStatus(), "assemblyStatus", packBean);
        }

        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.ASC.getValue()}, new String[]{"assemblyStatus"}, packBean);

        List<MesProductionAssembly> mesProductionAssemblies = mesProductionAssemblyRepository.findByHqlWhere(packBean);
        if (CollectionUtils.isEmpty(mesProductionAssemblies)) {
            MesPcnException.throwFlowException("装配件信息不存在");
        }
        return mesProductionAssemblies;
    }

    @Override
    public void reworkSuccess(MesReworkTaskRequestModel requestModel) {

        MesReworkTask mesReworkTask = getMesReworkTask(requestModel);
        /**
         * 获取返工单信息
         */
        List<MesReworkTaskDetail> reworkTaskDetails = getMesReworkTaskDetail(mesReworkTask);
        if (!CollectionUtils.isEmpty(reworkTaskDetails)) {
            MesPcnException.throwFlowException("还有位置未返工完成,请检查");
        }
        mesReworkTask.setStatus(MesExtEnumUtil.REWORK_TASK_STATUS.FINISH.getValue());
        mesReworkTaskRepository.save(mesReworkTask);
    }

    @Override
    public void assemblySnRepeat(MesReworkTaskRequestModel requestModel) {

        try {
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(requestModel.getOrganizeCode());
            DdlPreparedPack.getStringEqualPack(requestModel.getSn(), "productSn", packBean);
            MesProduceSn mesProduceSn = mesProduceSnRepository.getByProperty(packBean);
            if (mesProduceSn == null) {
                MesPcnException.throwFlowException("条码信息不存在");
            }
            requestModel.setAssemblyPartNo(mesProduceSn.getPartNo());
            requestModel.setAssemblyPartStatus(MesExtEnumUtil.ASSEMBLY_STATUS.ASSEMBLY_STATUS_10.getValue());
            List<MesProductionAssembly> mesProductionAssemblies = assemblyQuery(requestModel);
            if (CollectionUtils.isEmpty(mesProductionAssemblies)) {
                MesPcnException.throwFlowException("未匹配到该条码的零件号");
            }
            // 需要变更之前的记录
            MesProductionAssembly mesProductionAssembly = mesProductionAssemblies.get(0);
            mesProductionAssembly.setAssemblyStatus(MesExtEnumUtil.ASSEMBLY_STATUS.ASSEMBLY_STATUS_20.getValue());


            // 需要新增一条替换条码的记录
            MesProductionAssembly mesProductionRepeatAssembly = new MesProductionAssembly();
            BeanUtils.copyProperties(mesProductionAssembly, mesProductionRepeatAssembly);
            mesProductionRepeatAssembly.setAssemblySn(requestModel.getSn());
            mesProductionRepeatAssembly.setAssemblyStatus(MesExtEnumUtil.ASSEMBLY_STATUS.ASSEMBLY_STATUS_10.getValue());
            mesProductionRepeatAssembly.setId(null);

            mesProductionAssembly.setRepeatAssemblySn(requestModel.getSn());
            mesProductionAssemblyRepository.update(mesProductionAssembly);

            mesProductionAssemblyRepository.save(mesProductionRepeatAssembly);
            /**
             * 需要更新NC标记后进入的表明细状态为已处理
             */
            for (String detailId : requestModel.getDetailIds().split(",")) {
                reworkRecord(Long.valueOf(detailId));
            }
        } catch (Exception e) {
            log.error("替换条码异常", e);
        }

    }


    private MesPartInspection getMesPartInspection(MesReworkTask mesReworkTask) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesReworkTask.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesReworkTask.getSn(), "sn", packBean);

        MesPartInspection mesPartInspection = mesPartInspectionRepository.getByProperty(packBean);

        return mesPartInspection;
    }

    private List<MesPartInspectionDetail> getMesPartInspectionDetail(List<MesReworkTaskDetail> reworkTaskDetails) {

        List<Long> ids = reworkTaskDetails.stream().map(MesReworkTaskDetail::getPartInspectionDetailId).collect(Collectors.toList());

        DdlPackBean packBean = DdlPackBean.getDdlPackBean(reworkTaskDetails.get(0).getOrganizeCode());
        DdlPreparedPack.getInPackList(ids, "id", packBean);

        return mesPartInspectionDetailRepository.findByHqlWhere(packBean);
    }

    private MesPartTypePicture getMesPartTypePicture(MesReworkTask mesReworkTask) {
        DdlPackBean partBean = DdlPackBean.getDdlPackBean(mesReworkTask.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesReworkTask.getPartNo(), "partNo", partBean);
        MesPart mesPart = mesPartRepository.getByProperty(partBean);
        if (mesPart == null) {
            MesPcnException.throwFlowException(String.format("零件信息不存在,根据物料号查询[%s]", mesReworkTask.getPartNo()));
        }

        DdlPackBean pictureBean = DdlPackBean.getDdlPackBean(mesReworkTask.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(mesPart.getPartTypeCode(), "partTypeCode", pictureBean);
        MesPartTypePicture mesPartTypePicture = mesPartTypePictureRepository.getByProperty(pictureBean);
        if (mesPartTypePicture == null) {
            throw new ImppBusiException(String.format("图片信息不存在,根据物料号查询[%s]", mesReworkTask.getPartNo()));
        }
        return mesPartTypePicture;
    }

    private MesReworkTask getMesReworkTask(MesReworkTaskRequestModel requestModel) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(requestModel.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(requestModel.getSn(), "sn", packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.REWORK_TASK_STATUS.CREATE.getValue(), "status", packBean);

        MesReworkTask res = mesReworkTaskRepository.getByProperty(packBean);
        if (res == null) {
            throw new ImppBusiException("返工单信息不存在");
        }
        return res;
    }
    private List<MesReworkTaskDetail> getMesReworkTaskDetail(MesReworkTask mesReworkTask) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(mesReworkTask.getOrganizeCode());
        DdlPreparedPack.getNumEqualPack(mesReworkTask.getId(), "pid", packBean);
        DdlPreparedPack.getNumEqualPack(MesExtEnumUtil.REWORK_TASK_DETAIL_STATUS.REWORK_TASK_DETAIL_STATUS_10.getValue(), "status", packBean);

        List<MesReworkTaskDetail> res = mesReworkTaskDetailRepository.findByHqlWhere(packBean);

        return res;
    }
}
