package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.busi;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesAssemblyExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProdRuleCfgExtService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProdRuleContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionAssemblySortContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionPartContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesProdRuleNosortCfg;
import cn.estsh.i3plus.pojo.mes.repository.MesProdRuleNoSortCfgRepository;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Slf4j
@Service
public class MesProdRuleCfgExtService implements IMesProdRuleCfgExtService {

    @Autowired
    private IMesAssemblyExtService assemblyExtService;

    @Autowired
    private MesProdRuleNoSortCfgRepository prodRuleNoSortCfgRepository;

    //【排序线】获取加工规则(生产工单对应的装配件绑定记录)
    @Override
    public List<MesProdRuleContext> getProdRuleSortContextList(MesProdRuleContext prodRuleContext, List<MesProductionPartContext> productionPartContextList) {

        //【排序线】获取生产工单装配件绑定记录
        List<MesProductionAssemblySortContext> productionAssemblySortContextList = assemblyExtService.getProductionAssemblySortContextList(prodRuleContext, productionPartContextList.stream().filter(o -> null != o).map(MesProductionPartContext::getWorkOrderNo).collect(Collectors.toList()));

        //装配件绑定记录根据生产工单分组
        Map<String, List<MesProductionAssemblySortContext>> assemblyDataMap = groupProductionAssemblySortContextList(productionAssemblySortContextList);

        //实例化排序线加工规则与装配件数据记录
        List<MesProdRuleContext> prodRuleContextList = new ArrayList<>();
        productionPartContextList.forEach(o -> prodRuleContextList.add(createSortProdRuleContext(o, assemblyDataMap)));

        return prodRuleContextList;

    }

    //装配件绑定记录根据生产工单分组
    private Map<String, List<MesProductionAssemblySortContext>> groupProductionAssemblySortContextList(List<MesProductionAssemblySortContext> productionAssemblySortContextList) {
        return CollectionUtils.isEmpty(productionAssemblySortContextList) ? null :
                productionAssemblySortContextList.stream().filter(o -> null != o).collect(Collectors.groupingBy(MesProductionAssemblySortContext::getWorkOrderNo));
    }

    //实例化排序线加工规则与装配件数据记录
    private MesProdRuleContext createSortProdRuleContext(MesProductionPartContext productionPartContext, Map<String, List<MesProductionAssemblySortContext>> assemblyDataMap) {
        MesProdRuleContext prodRuleContext = new MesProdRuleContext(productionPartContext.getOrganizeCode()).foreignKey(productionPartContext.getForeignKey());
        if (!CollectionUtils.isEmpty(assemblyDataMap)) prodRuleContext.assemblyDataJson(assemblyDataMap.get(productionPartContext.getWorkOrderNo()));
        return prodRuleContext;
    }

    //【非排序线】获取产品加工规则
    @Override
    public List<MesProdRuleNosortCfg> getProdRuleNosortCfgList(MesProdRuleContext prodRuleContext) {
        if (StringUtils.isEmpty(prodRuleContext.getOrganizeCode()) || StringUtils.isEmpty(prodRuleContext.getEquipmentCode())) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(prodRuleContext.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(prodRuleContext.getEquipmentCode(), MesPcnExtConstWords.EQUIPMENT_CODE, packBean);
        DdlPreparedPack.getStringEqualPack(prodRuleContext.getOutPartNo(), MesPcnExtConstWords.OUT_PART_NO, packBean);

        if (prodRuleContext.getInPartNoIsEmpty()) DdlPreparedPack.getIsNull(MesPcnExtConstWords.IN_PART_NO, packBean);
        else DdlPreparedPack.getStringEqualPack(prodRuleContext.getInPartNo(), MesPcnExtConstWords.IN_PART_NO, packBean);

        if (prodRuleContext.getInPartNoRuleIsEmpty()) DdlPreparedPack.getIsNull(MesPcnExtConstWords.IN_PART_NO_RULE, packBean);
        else DdlPreparedPack.getIsNotNull(MesPcnExtConstWords.IN_PART_NO_RULE, packBean);

        return prodRuleNoSortCfgRepository.findByHqlWhere(packBean);
    }

    //【非排序线】获取产品加工规则对应的装配件信息
    @Override
    public MesProdRuleContext getProdRuleContext(MesProdRuleNosortCfg prodRuleNosortCfg, MesProductionPartContext productionPartContext) {
        MesProdRuleContext prodRuleContext = new MesProdRuleContext().foreignKey(productionPartContext.getForeignKey()).pid(prodRuleNosortCfg.getId());
        BeanUtils.copyProperties(prodRuleNosortCfg, prodRuleContext);
        //获取非排序装配件配置信息
        if (CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue() == prodRuleContext.getIsHasBind()) prodRuleContext.assemblyDataJson(assemblyExtService.getProductionAssemblyNosortContextList(prodRuleContext.getOrganizeCode(), prodRuleContext.getPid()));
        return prodRuleContext;
    }

}
