package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import cn.estsh.i3plus.ext.mes.pcn.api.base.IMesMouldMultiCavityService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesEquipVariableCfgRuleMatchDispatchService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesEquipmentLogExtService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesCellEquipContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesEquipVariableCfgCollectContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesEquipVariableCollectContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionProcessContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.util.MesPcnExtConstWords;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesEnumUtil;
import cn.estsh.i3plus.pojo.base.enumutil.MesPcnEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StationResultBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.repository.IMesMouldMappingCfgRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProdMouldRecordRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.*;

/**
 * @Description : 生成开模记录工步
 * @Author : zxw
 **/
@Slf4j
@Service("mesMouldRecordGenerateStepService")
public class MesMouldRecordGenerateStepService extends BaseStepService {

    @Autowired
    private MesProdMouldRecordRepository mesProdMouldRecordRepository;

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private IMesMouldMappingCfgRepository mesMouldMappingCfgRepository;
    @Autowired
    private IMesProductionDispatchContextStepService mesProductionDispatchContextStepService;

    @Override
    public StepResult execute(StationRequestBean reqBean) {

        StationResultBean resultBean = new StationResultBean();

        //获取上下文信息
        MesProductionProcessContext productionProcessContext = productionProcessContextStepService.getEquipmentVariableList(reqBean, MesExtEnumUtil.EQUIP_VARIABLE_TYPE.PRODUCTION.getValue());
        // 获取条码信息
        List<MesProduceSn> mesProduceSns = mesProductionDispatchContextStepService.getOutProduceSnDataContext(reqBean);
        // 获取头道模具号
        MesEquipVariableCollectContext mesFirstMouldNoCollectContext = mesProductionDispatchContextStepService.getFirstMouldNoContext(reqBean);
        // 获取模具号
        MesEquipVariableCollectContext mesMouldNoCollectContext = mesProductionDispatchContextStepService.getMouldNoContext(reqBean);

        List<MesProdMouldRecord> mesProdMouldRecords = new ArrayList<>();

        for (MesProduceSn mesProduceSn : mesProduceSns) {
            // 获取工位设备信息
            MesCellEquipContext mesCellEquipContext = productionProcessContext.getCurCellEquip();

            String equipmentCode = mesCellEquipContext.getEquipmentCode();

            String mouldNo = "";

            if (mesFirstMouldNoCollectContext != null) {
                mouldNo = mesFirstMouldNoCollectContext.getEquipVariableValue();
            } else if (mesFirstMouldNoCollectContext == null && mesMouldNoCollectContext != null) {
                mouldNo = mesMouldNoCollectContext.getEquipVariableValue();
            }
            DdlPackBean packBean = DdlPackBean.getDdlPackBean();
            DdlPreparedPack.getStringEqualPack(equipmentCode, "equipmentCode", packBean);
            DdlPreparedPack.getStringEqualPack(mouldNo, "mouldNo", packBean);
            MesMouldMappingCfg mesMouldMappingCfg = mesMouldMappingCfgRepository.getByProperty(packBean);
            //配置错误 抛出异常
            if (mesMouldMappingCfg == null) execExpSendMsgAndThrowEx(reqBean, resultBean,
                    String.format("请检查模具映射关系维护,根据设备[%s],模具号[%s]", equipmentCode, mouldNo));


            MesProdMouldRecord mesProductionRecord = getMesProdMouldRecord(mesCellEquipContext, mouldNo, mesMouldMappingCfg);
            mesProdMouldRecords.add(mesProductionRecord);
        }

        mesProdMouldRecordRepository.saveAll(mesProdMouldRecords);

        return StepResult.getSuccessComplete();
    }

    private MesProdMouldRecord getMesProdMouldRecord(MesCellEquipContext mesCellEquipContext, String mouldNo, MesMouldMappingCfg mesMouldMappingCfg) {
        MesProdMouldRecord mesProdMouldRecord = new MesProdMouldRecord();
        mesProdMouldRecord.setEquipId(mesCellEquipContext.getEquipId());
        mesProdMouldRecord.setMeterName(mesMouldMappingCfg.getMeterName());
        mesProdMouldRecord.setAssetNum(mesMouldMappingCfg.getAssetNum());
        mesProdMouldRecord.setEquipmentCode(mesCellEquipContext.getEquipmentCode());
        return mesProdMouldRecord;
    }
}
