package cn.estsh.i3plus.ext.mes.pcn.apiservice.serviceimpl.step;

import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionDispatchContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.api.busi.IMesProductionProcessContextStepService;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesEquipVariableCollectContext;
import cn.estsh.i3plus.ext.mes.pcn.pojo.context.MesProductionProcessContext;
import cn.estsh.i3plus.mes.pcn.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.mes.pcn.serviceimpl.fsm.BaseStepService;
import cn.estsh.i3plus.pojo.base.codemaker.SnowflakeIdMaker;
import cn.estsh.i3plus.pojo.base.enumutil.MesEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesPart;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.MesProductionRecord;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCell;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.model.StationRequestBean;
import cn.estsh.i3plus.pojo.mes.model.StepResult;
import cn.estsh.i3plus.pojo.mes.repository.MesPartRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProduceSnRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProductionRecordRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.hutool.core.date.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @Description : 生成加工记录工步
 * @Author : wangjie
 **/
@Slf4j
@Service
public class MesProductionReocrdGenerateStepService extends BaseStepService {
    @Autowired
    private SnowflakeIdMaker snowflakeIdMaker;

    @Autowired
    private IMesProductionProcessContextStepService productionProcessContextStepService;

    @Autowired
    private MesProduceSnRepository mesProduceSnRepository;

    @Autowired
    private MesProductionRecordRepository mesProductionRecordRepository;

    @Autowired
    private IMesProductionDispatchContextStepService mesProductionDispatchContextStepService;

    public static final String ORDER_NO_JIS_SORT = "ORDER_NO_JIS_SORT";


    @Override
    public StepResult execute(StationRequestBean reqBean) {

        MesProductionProcessContext productionProcessContext = productionProcessContextStepService.getEquipmentVariableList(reqBean, MesExtEnumUtil.EQUIP_VARIABLE_TYPE.PRODUCTION.getValue());
        List<MesProduceSn> mesProduceSns = mesProductionDispatchContextStepService.getOutProduceSnDataContext(reqBean);

        List<MesProductionRecord> mesProductionRecords = new ArrayList<>();
        mesProduceSns.forEach(mesProduceSn -> {

            MesProductionRecord mesProductionRecord = getMesProductionRecord(productionProcessContext, mesProduceSn, reqBean);
            mesProductionRecords.add(mesProductionRecord);
        });

        mesProductionRecordRepository.saveAll(mesProductionRecords);

        return StepResult.getSuccessComplete();
    }

    private MesProductionRecord getMesProductionRecord(MesProductionProcessContext productionProcessContext, MesProduceSn mesProduceSn, StationRequestBean reqBean) {
        MesProductionRecord mesProductionRecord = new MesProductionRecord();

        mesProductionRecord.setSerialNumber(mesProduceSn.getSerialNumber());
        mesProductionRecord.setAreaCode(mesProduceSn.getAreaCode());
        mesProductionRecord.setCraftCode(mesProduceSn.getCraftCode());
        mesProductionRecord.setCustSn(mesProduceSn.getCustSn());
        mesProductionRecord.setEquipmentCode(mesProduceSn.getEquipmentCode());
        mesProductionRecord.setIsComplete(MesEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        mesProductionRecord.setCompleteDateTime(DateUtil.formatDateTime(new Date()));
        mesProductionRecord.setLotNo(mesProduceSn.getLotNo());
        mesProductionRecord.setEquipmentName(mesProduceSn.getEquipmentName());

        String mouldNo = "";
        MesEquipVariableCollectContext mesFirstMouldNoCollectContext = mesProductionDispatchContextStepService.getFirstMouldNoContext(reqBean);
        MesEquipVariableCollectContext mesMouldNoCollectContext = mesProductionDispatchContextStepService.getMouldNoContext(reqBean);

        if (mesFirstMouldNoCollectContext != null) {
            mouldNo = mesFirstMouldNoCollectContext.getEquipVariableValue();
        } else if (mesFirstMouldNoCollectContext == null && mesMouldNoCollectContext != null) {
            mouldNo = mesMouldNoCollectContext.getEquipVariableValue();
        }
        mesProductionRecord.setMouldNo(mouldNo);
        // todo
        // mesProductionRecord.setMouldRecordId();
        mesProductionRecord.setProcessCode(mesProduceSn.getProcessCode());
        mesProductionRecord.setReportPartNo(mesProduceSn.getPartNo());
        mesProductionRecord.setReportStatus(MesExtEnumUtil.REPORT_STATUS.REPORT_STATUS_10.getValue());
        mesProductionRecord.setReportType(mesProduceSn.getReportType() + "");
        mesProductionRecord.setShiftCode(mesProduceSn.getShiftCode());
        mesProductionRecord.setShiftGroup(mesProduceSn.getShiftGroup());
        mesProductionRecord.setWorkOrderNo(mesProduceSn.getWorkOrderNo());
        mesProductionRecord.setWorkCellCode(mesProduceSn.getWorkCellCode());
        mesProductionRecord.setWorkCenterCode(mesProduceSn.getWorkCenterCode());
        return mesProductionRecord;
    }

}
